#
# facade.rb
#
#   Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#

require 'amstd/must'


module Racc

  Version = '0.11.1'

class Compiler

  attr :class_name

  attr :parser
  attr :ruletable
  attr :tokentable
  attr :statetable
  attr :formatter
  attr :interf

  attr :logic
  attr :srconf
  attr :rrconf

  attr :dsrc
  
  attr :debug
  attr :d_prec
  attr :d_rule
  attr :d_token
  attr :d_state
  attr :d_reduce
  attr :d_shift


  def initialize( debugopt )
    @dsrc     = debugopt[ 'debug-src' ]

    @debug    = debugopt[ 'debug' ]
    @d_prec   = debugopt[ 'prec' ]
    @d_rule   = debugopt[ 'rule' ]
    @d_token  = debugopt[ 'token' ]
    @d_state  = debugopt[ 'state' ]
    @d_reduce = debugopt[ 'reduce' ]
    @d_shift  = debugopt[ 'shift' ]

    @logic  = []
    @rrconf = []
    @srconf = []
    @tokentable = TokenTable.new( self )
    @ruletable  = RuleTable.new( self )
    @interf     = BuildInterface.new( self )
    @parser     = RaccParser.new( self )
    @statetable = LALRstateTable.new( self )
    @codegen    = AListTableGenerator.new( self )
    @verbosegen = VerboseOutputFormatter.new( self )
  end

  def compile( str, fn = '' )
    parse( str, fn )
    init_rule
    init_state
    resolve
  end

  def parse( str, fname = '' )
    @parser.parse( str, fname )
    @class_name = @parser.classname
  end

  def init_rule
    @ruletable.do_initialize
  end

  def init_state
    @statetable.do_initialize
  end

  def resolve
    @statetable.resolve
  end

  def source( f = '' )
    @codegen.output f
    f
  end

  def output( f = '' )
    @verbosegen.output f
    f
  end

end


end   # module Racc
